
#include <stddef.h>                     // Defines NULL
#include <stdbool.h>                    // Defines true
#include <stdlib.h>                     // Defines EXIT_FAILURE
#include "system/common/sys_module.h"   // SYS function prototypes
#include "system_config.h"
#include "system_definitions.h"
#include "pragma.h"

SYSTEM_OBJECTS sysObj;


#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"

#define STARTTASK_PRIO 3
#define STARTTASK_DELAY 100
#define STARTTASK_STACKSIZE 128

#define LEDTASK1_PRIO 3
#define LEDTASK1_DELAY 1000
#define LEDTASK1_STACKSIZE 128

#define LEDTASK2_PRIO 4
#define LEDTASK2_DELAY 55
#define LEDTASK2_STACKSIZE 128

static void StartTask(void *pvParameters );
static void LEDblink1(void *pvParameters );
static void LEDblink2(void *pvParameters );



SemaphoreHandle_t  LEDMutex;

#ifdef TRC_USE_TRACEALYZER_RECORDER
traceString LED1Chn;
#endif










volatile int n;

int main ( void )
{
    SYS_CLK_Initialize( NULL );
    SYS_DEVCON_Initialize(SYS_DEVCON_INDEX_0, (SYS_MODULE_INIT*)NULL);
    SYS_DEVCON_PerformanceConfig(SYS_CLK_SystemFrequencyGet());
    SYS_PORTS_Initialize();
    
    
#ifdef TRC_USE_TRACEALYZER_RECORDER
#if (TRC_CFG_RECORDER_MODE == TRC_RECORDER_MODE_SNAPSHOT) 
    vTraceEnable(TRC_START);
#else
    vTraceEnable(TRC_INIT); 
#endif  
#endif

    BSP_Initialize();        
    SYS_INT_Initialize();
    
    LEDMutex = xSemaphoreCreateMutex();
    
	xTaskCreate( StartTask, "Startup", STARTTASK_STACKSIZE, NULL, STARTTASK_PRIO, NULL );
    
    
   
    vStartDemoApplication();
    vTaskStartScheduler();
    while (1)
    {
    }
    

    while ( true )
    {
         BSP_LEDOn(BSP_LED_1);
         BSP_LEDOn(BSP_LED_2);
         BSP_LEDOn(BSP_LED_3);
         
         BSP_LEDOff(BSP_LED_1);
         BSP_LEDOff(BSP_LED_2);
         BSP_LEDOff(BSP_LED_3);
    }

    /* Execution should not come here during normal operation */

    return ( EXIT_FAILURE );
}

static void StartTask(void *pvParameters )
{
    /* Remove compiler warning about unused parameter. */
	( void ) pvParameters;
    int n;
    

    xTaskCreate( LEDblink1, "LED1", LEDTASK1_STACKSIZE, NULL, LEDTASK1_PRIO, NULL );
	xTaskCreate( LEDblink2, "LED2", LEDTASK2_STACKSIZE, NULL, LEDTASK2_PRIO, NULL );
    

    for(n=0;n<5;n++)
    {
        vTaskDelay(STARTTASK_DELAY);
        BSP_LEDOn(BSP_LED_2);
        vTaskDelay(STARTTASK_DELAY);
        BSP_LEDOff(BSP_LED_2);       
    }
    vStartDemoApplication();
    vTaskDelete( 0 );
}

static void LEDblink1(void *pvParameters )
{
	/* Remove compiler warning about unused parameter. */
	( void ) pvParameters;
    
    uint32_t * mem;
   
    while(1)
	{
		xSemaphoreTake(LEDMutex,portMAX_DELAY );
        BSP_LEDOn(BSP_LED_1);
        mem=pvPortMalloc(4);
        #if (TRC_USE_TRACEALYZER_RECORDER == 1) 
            vTracePrintF(LED1Chn, "LED:%d",1);
        #endif
		vTaskDelay(LEDTASK1_DELAY);
        BSP_LEDOff(BSP_LED_1);
        vPortFree(mem);
		xSemaphoreGive(LEDMutex);
        #if (TRC_USE_TRACEALYZER_RECORDER == 1) 
            vTracePrintF(LED1Chn, "LED:%d",0);
        #endif
		vTaskDelay(LEDTASK1_DELAY);    
	}
}

static void LEDblink2(void *pvParameters )
{
	/* Remove compiler warning about unused parameter. */
	( void ) pvParameters;
    
    uint32_t * mem;
   
    while(1)
	{
		xSemaphoreTake(LEDMutex,portMAX_DELAY );
        BSP_LEDOn(BSP_LED_1);
        mem=pvPortMalloc(4);
        #if (TRC_USE_TRACEALYZER_RECORDER == 1) 
            vTracePrintF(LED1Chn, "LED:%d",1);
        #endif
		vTaskDelay(LEDTASK2_DELAY);
        BSP_LEDOff(BSP_LED_1);
        vPortFree(mem);
		xSemaphoreGive(LEDMutex);
        #if (TRC_USE_TRACEALYZER_RECORDER == 1) 
            vTracePrintF(LED1Chn, "LED:%d",0);
        #endif
		vTaskDelay(LEDTASK2_DELAY);    
	}
}


/*******************************************************************************
 End of File
*/

